
#include "inc.h"

#include "lib.h"
#include "printf.h"
#include "iwram_arm.h"
#include "div_arm.h"
#include "rand.h"

#include "main.h"


//Mode3
//SPR J[\

//---------------------------------------------------------------------------
#define STATE_INIT 					1		//is
#define STATE_RESET					2
#define STATE_GAME					3
#define STATE_INPUT					4
#define STATE_ANSWER				5

#define GOTAKU_LOAD_HEADER_OFFSET	0
#define GOTAKU_LOAD_HEADER_SIZE		256
#define GOTAKU_LOAD_QUESTION_OFFSET	2048
#define GOTAKU_LOAD_QUESTION_SIZE	256

#define GOTAKU_MAX_HEADER_CNT		8
#define GOTAKU_COLOR_BACKGROUND		RGB(0,0,15)

#define CURSOR_TILENO				514
//---------------------------------------------------------------------------
typedef struct {
	u8   x;
	u8   y;									//\̍s
	u8   wy;								//̍s
	u8   cx;
	u8   cy;
	s16  minX;								//ړ
	s16  maxX;
	s16  minY;
	s16  maxY;
	u16  sprNo;
	u16  sprTile;
	u16  sprSize;
	u16  sprForm;
	u16  key;								//ړp
	u8   selPoint;							//I
} ST_CURSOR;


typedef struct {
	u8  type[16];							//W
	u16 pass;								//gp
	u16 questionCnt;						//W̖萔iubNƓj
	u16 skip;								//ANZX̂߂ɓǂݔ΂ubN
	u8  file[12];							//v[[f[^t@C
	u8  code[8];							//}WbNR[hi5TAKUQDT or 5TAKUQDXj
//	u8  fill[214];							//ubNACg̒pl߂
} ST_GOTAKU_HEADER;


typedef struct {
	u8 str[116];							//
    u8 choice[5][28];						//IivTj
} ST_GOTAKU_QUESTION;


typedef struct {
	u8* pStart;
	u8* pQuestion;
	u16 maxQuestionCnt;

	u16 questionNo;
	u8* pChoice[5];
    u8  answer;

	ST_GOTAKU_HEADER   header[GOTAKU_MAX_HEADER_CNT];
	ST_GOTAKU_QUESTION question;
} ST_GOTAKU;


//---------------------------------------------------------------------------
ST_CURSOR   Cursor;
ST_GOTAKU   Gotaku;

u8 State;								//is󋵃tO

//---------------------------------------------------------------------------
EWRAM_CODE int main()
{
	State = STATE_INIT;

	for(;;)
	{
		switch(State)
		{
			case STATE_INIT:    StateInit();    break;
			case STATE_RESET:   StateReset();   break;
			case STATE_GAME:    StateGame();    break;
			case STATE_INPUT:   StateInput();   break;
			case STATE_ANSWER:  StateAnswer();  break;

			default:
				DebugMessage("Err: Main State");
				break;
		}
	}

	return 0;
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateInit()
{
	InitWS();
	InitKey();
	InitSprite();
	InitFont();
	InitIRQ();

	State = STATE_RESET;
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitWS()
{
	//J[hbW̕Wݒ
	REG_WSCNT = 0x4317;
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitKey()
{
	KeyInit();
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitSprite()
{
	SpriteInit();

	SpriteSetPalData16((u16*)&sprPal);
	SpriteSetData ((u16*)&sprTiles, SPR_IMG_BYTE);
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitFont()
{
	Mode3SetFont((u8*)&shnmk12Bitmap, SHNMK12_IMG_CX, SHNMK12_FONT_CX, SHNMK12_FONT_CY,
	             (u16*)&shnm_fnt_txt, SHNM_FNT_FONT_CNT);
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitIRQ()
{
	REG_INTERUPT = (u32)IRQUserHandler;
	REG_IE       = IRQ_BIT_TIMER0;

	REG_TM0D     = 65536 - 2621;			// (((16*1024*1024) / 64) * 10) / 1000
	REG_TM0CNT   = TM_ENABLE | TM_FREQ_PER_64 | TM_USEIRQ;

	REG_IME      = IRQ_MASTER_ON;
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateReset()
{
	GoTakuInit();
	CursorInit();

	GoTakuDrawFrame();
	SetMode(MODE_3 | BG2_ENABLE | OBJ_ENABLE | OBJ_MAP_2D);

	if( GoTakuLoadFile() == FALSE )
	{
		GoTakuDrawFont(1, 2, (u8*)"Tspf[^܂łB");
		for(;;){}
	}

	State = STATE_GAME;
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateGame()
{
	//NA
	Mode3DrawFillBox(12, 24, 240-12-12+1, 60+1, GOTAKU_COLOR_BACKGROUND);
	Mode3DrawFillBox(48, 96,     12*15+1, 60+1, GOTAKU_COLOR_BACKGROUND);

oneMore:

	GoTakuSetQuestion();
	GoTakuSetRandomAnswer();

	//񂪒́AI΂Ȃ悤ɂ܂
	if( GoTakuChkStrLength() == FALSE )
	{
		goto oneMore;
	}

	GoTakuDrawQuestionNo();
	GoTakuDrawQuestion();
	CursorDraw();

	State = STATE_INPUT;
}
//---------------------------------------------------------------------------
EWRAM_CODE void GoTakuInit()
{
	ST_GOTAKU* g = &Gotaku;

	g->pStart         = NULL;
	g->pQuestion      = NULL;
	g->maxQuestionCnt = 0;
	g->questionNo     = 0;
    g->answer         = 0;


	u8* p1 = (u8*)&Gotaku.header;
	u8* p2 = (u8*)&Gotaku.question;
	u16 i;

	for(i=0; i<5; i++)
	{
		g->pChoice[i] = g->question.choice[i];
	}

	for(i=0; i<sizeof(ST_GOTAKU_HEADER) * GOTAKU_MAX_HEADER_CNT; i++)
	{
		*p1++ = 0x00;
	}

	for(i=0; i<sizeof(ST_GOTAKU_QUESTION); i++)
	{
		*p2++ = 0x00;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE bool GoTakuChkStrLength()
{
	ST_GOTAKU* g = &Gotaku;
	u16 i, len;

	//I̕񂾂`FbN܂
	for(i=0; i<5; i++)
	{
		len = _Length(g->question.choice[i]);
		
		if(len > 15*2 + 1)
		{
			return FALSE;
		}
	}

	return TRUE;
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateInput()
{
	WaitForVsync();

	ST_CURSOR* p = &Cursor;
	p->key       = KeyGet();

	if(p->key == KEY_NONE)
	{
		return;
	}
	if(p->key == KEY_A)
	{
		State = STATE_ANSWER;
		return;
	}

	CursorInput();
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateAnswer()
{
	WaitForVsync();

	ST_CURSOR* c = &Cursor;
	ST_GOTAKU* g = &Gotaku;

	//NA
	Mode3DrawFillBox(12, 24, 240-12-12+1, 60+1, GOTAKU_COLOR_BACKGROUND);
	Mode3DrawFillBox(48, 96,     12*15+1, 60+1, GOTAKU_COLOR_BACKGROUND);

	if(g->answer == c->selPoint)
	{
		GoTakuDrawFont(8, 4, (u8*)"@I");
	}
	else
	{
		GoTakuDrawFont(8, 4, (u8*)"sI");
	}
	GoTakuDrawFont(4, 8+g->answer, (u8*)g->pChoice[g->answer]);
	WaitMs(0x0400);

	State = STATE_GAME;
}
//---------------------------------------------------------------------------
EWRAM_CODE bool GoTakuLoadFile()
{
	if(GoTakuChkSignature() == FALSE)
	{
		return FALSE;
	}

	GoTakuSetHeader();
	GoTakuSetRand();

	return TRUE;
}
//---------------------------------------------------------------------------
EWRAM_CODE bool GoTakuChkSignature()
{
	static u8 sigStr[2][8] = { { '5','T','A','K','U','Q','D','T' }, { '5','T','A','K','U','Q','D','X' } };
	u8  sigCnt    = 2;
	u8  sigLength = 8;

	u8* p = EtfsGetPointer() + 16 + 2 + 2 + 2 + 12;
	u8* tp;
	u16 i,j;

	for(i=0; i<sigCnt; i++)
	{
		tp = p;
		for(j=0; j<sigLength; j++)
		{
			if(sigStr[i][j] != *p++)
			{
				break;
			}
		}

		if(j == sigLength)
		{
			return TRUE;
		}
	}

	return FALSE;
}
//---------------------------------------------------------------------------
EWRAM_CODE void GoTakuSetHeader()
{
	ST_GOTAKU* g = &Gotaku;

	g->pStart    = EtfsGetPointer();
	g->pQuestion = EtfsGetPointer() + GOTAKU_LOAD_QUESTION_OFFSET;

	u16 i,j;
	u8* p;
	u8* h;
	for(i=0; i<GOTAKU_MAX_HEADER_CNT; i++)
	{
		p = (u8*)(g->pStart + GOTAKU_LOAD_HEADER_SIZE * i);
		h = (u8*)(&g->header[i]);

		for(j=0; j<sizeof(ST_GOTAKU_HEADER); j++)
		{
			*h++ = *p++;
		}
		g->maxQuestionCnt += g->header[i].questionCnt;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void GoTakuSetRand()
{
	GoTakuDrawFont(1, 3, (u8*)"f[^̓ǂݎɐ܂B\n`{^ĂB");
	KeyWait(KEY_A);
}
//---------------------------------------------------------------------------
EWRAM_CODE void GoTakuDrawFont(u16 sx, u16 sy, u8* str)
{
	u16 ex  = 19;						//܂Ԃ
	u16 len = _Length(str);

	u16 i = 0;
	u16 x = 0;
	u16 y = 0;
	u8  chr[3];

	while(i <= len)
	{
		chr[0] = str[i++];
		chr[1] = str[i++];
		chr[2] = '\0';

		Mode3DrawFontBackGround(sx+x, sy+y, chr, COLOR_WHITE, GOTAKU_COLOR_BACKGROUND);

		x++;
		if(sx+x >= ex || str[i] == '\n')
		{
			x = 0;
			y++;

			if(str[i] == '\n')
			{
				i++;
			}
		}
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void GoTakuSetQuestion()
{
	ST_GOTAKU* g = &Gotaku;

	u16 no = _UMod(_Rand(), g->maxQuestionCnt);
	u8* p  = g->pQuestion + GOTAKU_LOAD_QUESTION_SIZE * no;
	u8* q  = (u8*)&g->question;

	u16 i;
	//fR[h
	for(i=0; i<sizeof(ST_GOTAKU_QUESTION); i++)
	{
		*q++ = *p++ ^ 0x80;
	}

	//̏I[L
	GoTakuModStr(g->question.str, 116);

	for(i=0; i<5; i++)
	{
		GoTakuModStr(g->question.choice[i], 28);
	}

	g->questionNo = no;
}
//---------------------------------------------------------------------------
#define IsSJIS(c) (((c)>=0x81 && (c)<=0x9f) || ((c)>=0xe0 && (c)<=0xef))

EWRAM_CODE void GoTakuModStr(u8* str, u16 cnt)
{
	u16 i = 0;

	while(i < cnt)
	{
		if( IsSJIS(str[i]) != TRUE )
		{
			if(str[i] == 0xa0)
			{
				str[i] = 0x00;
				return;
			}
		}

		i+=2;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void GoTakuSetRandomAnswer()
{
	ST_GOTAKU* g = &Gotaku;

	u16 max = _UMod(_Rand(), 0x40);
	u16 swap1;
	u16 swap2;
	u8* tmp;
	u16 i;

	//ݒɖ߂܂
    g->answer = 0;
	for(i=0; i<5; i++)
	{
		g->pChoice[i] = g->question.choice[i];
	}

	for(i=0; i<max; i++)
	{
		swap1 = _UMod(_Rand(), 5);
		swap2 = _UMod(_Rand(), 5);
		if(swap1 == swap2)
		{
			continue;
		}

		//ւ
		tmp = g->pChoice[swap1];
		g->pChoice[swap1] = g->pChoice[swap2];
		g->pChoice[swap2] = tmp;

		if(swap1 == g->answer)
		{
			g->answer = swap2;
		}
		else if(swap2 == g->answer)
		{
			g->answer = swap1;
		}
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void GoTakuDrawFrame()
{
	Mode3DrawCls(GOTAKU_COLOR_BACKGROUND);

	GoTakuDrawFont(1, 0, (u8*)"|@@@|");

	//u藓v̕\
	Mode3DrawBox(12-1, 24-1, 240-12-12+2, 60+2, COLOR_WHITE);

	//u񓚗v̕\
	Mode3DrawBox(12-1, 96-1, 12*18+2, 60+2, COLOR_WHITE);
	GoTakuDrawFont(2,  8, (u8*)"PD");
	GoTakuDrawFont(2,  9, (u8*)"QD");
	GoTakuDrawFont(2, 10, (u8*)"RD");
	GoTakuDrawFont(2, 11, (u8*)"SD");
	GoTakuDrawFont(2, 12, (u8*)"TD");
}
//---------------------------------------------------------------------------
EWRAM_CODE void GoTakuDrawQuestion()
{
	ST_GOTAKU* g = &Gotaku;

	GoTakuDrawFont(1, 2, (u8*)g->question.str);

	u16 i;
	for(i=0; i<5; i++)
	{
		GoTakuDrawFont(4, 8+i, (u8*)g->pChoice[i]);
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void GoTakuDrawQuestionNo()
{
	ST_GOTAKU* g = &Gotaku;

	//0x82 0x4f = SJIŚuOv
	static u8 conStr[7] = { 0x82, 0x4f, 0x82, 0x4f, 0x82, 0x4f, 0x00 };
	u8 str[7];

	u16 i=0;
	for(i=0; i<7; i++)
	{
		str[i] = conStr[i];
	}

	u16 qNo = g->questionNo;
	u16 no;
	u8 keta1, keta2, keta3;

	no = _UDiv(qNo, 100); keta3 = no; qNo -= no*100;
	no = _UDiv(qNo, 10);  keta2 = no; qNo -= no*10;
	no = _UMod(qNo, 10);  keta1 = no; 

	str[1] += keta3;
	str[3] += keta2;
	str[5] += keta1;

	GoTakuDrawFont(3, 0, (u8*)str);
}


//---------------------------------------------------------------------------
EWRAM_CODE void CursorInit()
{
	ST_CURSOR* p = &Cursor;

	p->x         = 1;
	p->y         = 8;
	p->wy        = 0;
	p->minX      = 1;
	p->maxX      = 1;
	p->minY      = 8;
	p->maxY      = 8+4;
	p->cx        = 12;
	p->cy        = 12;
	p->sprNo     = 0;
	p->sprTile   = CURSOR_TILENO;
	p->sprSize   = SP_SIZE_16;
	p->sprForm   = SP_SQUARE;
	p->key       = KEY_NONE;
	p->selPoint  = 0;

	SpriteSetChr (p->sprNo, p->sprTile);
	SpriteSetSize(p->sprNo, p->sprSize, p->sprForm, SP_COLOR_16);
}
//---------------------------------------------------------------------------
EWRAM_CODE void CursorDraw()
{
	ST_CURSOR* p = &Cursor;
	u16 x = p->x * p->cx;
	u16 y = p->y * p->cy;

	SpriteMove(p->sprNo, x, y);
}
//---------------------------------------------------------------------------
EWRAM_CODE void CursorInput()
{
	ST_CURSOR* p = &Cursor;

	if(p->key & KEY_UP  ) p->key |= KEY_UP;
	if(p->key & KEY_DOWN) p->key |= KEY_DOWN;
	if(p->key & KEY_A   ) p->key |= KEY_A;

	if( CursorIsMove() == TRUE )
	{
		CursorMove();
		CursorDraw();

		p->selPoint = p->y - p->minY;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE bool CursorIsMove()
{
	ST_CURSOR* p = &Cursor;

	s16 tx1 = p->x;
	s16 ty1 = p->y;

	if(p->key & KEY_UP   ) ty1 -= 1;
	if(p->key & KEY_DOWN ) ty1 += 1;
	if(p->key & KEY_LEFT ) tx1 -= 1;
	if(p->key & KEY_RIGHT) tx1 += 1;

	if(tx1< p->minX || tx1>p->maxX) return FALSE;
	if(ty1< p->minY || ty1>p->maxY) return FALSE;

	return TRUE;
}
//---------------------------------------------------------------------------
EWRAM_CODE void CursorMove()
{
	ST_CURSOR* p = &Cursor;

	if(p->key & KEY_UP   ) p->y -= 1;
	if(p->key & KEY_DOWN ) p->y += 1;
	if(p->key & KEY_LEFT ) p->x -= 1;
	if(p->key & KEY_RIGHT) p->x += 1;
}
